# This file was automatically generated by SWIG
package Solaris::DeviceTree::Libdevinfo::Impl;
require Exporter;
require DynaLoader;
@ISA = qw(Exporter DynaLoader);
package Solaris::DeviceTree::Libdevinfo::Impl;
bootstrap Solaris::DeviceTree::Libdevinfo::Impl;
package Solaris::DeviceTree::Libdevinfo::Impl;
@EXPORT = qw( di_init di_fini di_parent_node di_sibling_node di_child_node di_drv_first_node di_drv_next_node $NULL $DI_NODE_NIL $DI_MINOR_NIL $DI_PROM_PROP_NIL $DI_PROM_HANDLE_NIL $DIIOC $DINFOSUBTREE $DINFOMINOR $DINFOPROP $DINFOCPYALL di_binding_name di_bus_addr di_devid di_driver_name di_state $DI_DRIVER_DETACHED $DI_DEVICE_OFFLINE $DI_DEVICE_DOWN $DI_BUS_QUIESCED $DI_BUS_DOWN isDevidNull $DI_BUS_OPS $DI_CB_OPS $DI_STREAM_OPS di_driver_ops di_instance $DI_PSEUDO_NODEID $DI_SID_NODEID $DI_PROM_NODEID di_nodeid di_node_name di_prop_next di_prop_name di_prop_type $DI_PROP_TYPE_BOOLEAN $DI_PROP_TYPE_INT $DI_PROP_TYPE_STRING $DI_PROP_TYPE_BYTE $DI_PROP_TYPE_UNKNOWN $DI_PROP_TYPE_UNDEF_IT di_prop_devt di_prop_ints di_prop_strings di_prop_bytes di_minor_next di_minor_devt di_minor_name di_minor_nodetype di_minor_spectype $S_IFCHR $S_IFBLK di_prom_init di_prom_fini di_prom_prop_next di_prom_prop_name di_prom_prop_data di_devfs_path sdi_prop_devt devt_majorminor di_compatible_names isDDI_DEV_T_NONE newStringHandle getIndexedString freeStringHandle newIntHandle getIndexedInt freeIntHandle newUCharTHandle getIndexedByte freeUCharTHandle UCharTString isDI_NODE_NIL isDI_PROP_NIL getProp makeDI_PROP_NIL makeDI_MINOR_NIL isDI_MINOR_NIL makeDI_PROM_PROP_NIL isDI_PROM_PROP_NIL isDI_PROM_HANDLE_NIL isDDI_DEVID_NIL );
1;
