/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 1.3.19
 * 
 * This file is not intended to be easily readable and contains a number of 
 * coding conventions designed to improve portability and efficiency. Do not make
 * changes to this file unless you know what you are doing--modify the SWIG 
 * interface file instead. 
 * ----------------------------------------------------------------------------- */

/***********************************************************************
 * common.swg
 *
 *     This file contains generic SWIG runtime support for pointer
 *     type checking as well as a few commonly used macros to control
 *     external linkage.
 *
 * Author : David Beazley (beazley@cs.uchicago.edu)
 *
 * Copyright (c) 1999-2000, The University of Chicago
 * 
 * This file may be freely redistributed without license or fee provided
 * this copyright message remains intact.
 ************************************************************************/

#include <string.h>

#if defined(_WIN32) || defined(__WIN32__)
#       if defined(_MSC_VER)
#               if defined(STATIC_LINKED)
#                       define SWIGEXPORT(a) a
#                       define SWIGIMPORT(a) extern a
#               else
#                       define SWIGEXPORT(a) __declspec(dllexport) a
#                       define SWIGIMPORT(a) extern a
#               endif
#       else
#               if defined(__BORLANDC__)
#                       define SWIGEXPORT(a) a _export
#                       define SWIGIMPORT(a) a _export
#               else
#                       define SWIGEXPORT(a) a
#                       define SWIGIMPORT(a) a
#               endif
#       endif
#else
#       define SWIGEXPORT(a) a
#       define SWIGIMPORT(a) a
#endif

#ifdef SWIG_GLOBAL
#define SWIGRUNTIME(a) SWIGEXPORT(a)
#else
#define SWIGRUNTIME(a) static a
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef void *(*swig_converter_func)(void *);
typedef struct swig_type_info *(*swig_dycast_func)(void **);

typedef struct swig_type_info {
  const char             *name;                 
  swig_converter_func     converter;
  const char             *str;
  void                   *clientdata;	
  swig_dycast_func        dcast;
  struct swig_type_info  *next;
  struct swig_type_info  *prev;
} swig_type_info;

#ifdef SWIG_NOINCLUDE

SWIGIMPORT(swig_type_info *) SWIG_TypeRegister(swig_type_info *);
SWIGIMPORT(swig_type_info *) SWIG_TypeCheck(char *c, swig_type_info *);
SWIGIMPORT(void *)           SWIG_TypeCast(swig_type_info *, void *);
SWIGIMPORT(swig_type_info *) SWIG_TypeDynamicCast(swig_type_info *, void **);
SWIGIMPORT(const char *)     SWIG_TypeName(const swig_type_info *);
SWIGIMPORT(swig_type_info *) SWIG_TypeQuery(const char *);
SWIGIMPORT(void)             SWIG_TypeClientData(swig_type_info *, void *);

#else

static swig_type_info *swig_type_list = 0;

/* Register a type mapping with the type-checking */
SWIGRUNTIME(swig_type_info *)
SWIG_TypeRegister(swig_type_info *ti)
{
  swig_type_info *tc, *head, *ret, *next;
  /* Check to see if this type has already been registered */
  tc = swig_type_list;
  while (tc) {
    if (strcmp(tc->name, ti->name) == 0) {
      /* Already exists in the table.  Just add additional types to the list */
      if (tc->clientdata) ti->clientdata = tc->clientdata;	
      head = tc;
      next = tc->next;
      goto l1;
    }
    tc = tc->prev;
  }
  head = ti;
  next = 0;

  /* Place in list */
  ti->prev = swig_type_list;
  swig_type_list = ti;

  /* Build linked lists */
 l1:
  ret = head;
  tc = ti + 1;
  /* Patch up the rest of the links */
  while (tc->name) {
    head->next = tc;
    tc->prev = head;
    head = tc;
    tc++;
  }
  if (next) next->prev = head;  /**/
  head->next = next;
  return ret;
}

/* Check the typename */
SWIGRUNTIME(swig_type_info *) 
SWIG_TypeCheck(char *c, swig_type_info *ty)
{
  swig_type_info *s;
  if (!ty) return 0;        /* Void pointer */
  s = ty->next;             /* First element always just a name */
  do {
    if (strcmp(s->name,c) == 0) {
      if (s == ty->next) return s;
      /* Move s to the top of the linked list */
      s->prev->next = s->next;
      if (s->next) {
	s->next->prev = s->prev;
      }
      /* Insert s as second element in the list */
      s->next = ty->next;
      if (ty->next) ty->next->prev = s;
      ty->next = s;
      s->prev = ty;  /**/
      return s;
    }
    s = s->next;
  } while (s && (s != ty->next));
  return 0;
}

/* Cast a pointer up an inheritance hierarchy */
SWIGRUNTIME(void *) 
SWIG_TypeCast(swig_type_info *ty, void *ptr) 
{
  if ((!ty) || (!ty->converter)) return ptr;
  return (*ty->converter)(ptr);
}

/* Dynamic pointer casting. Down an inheritance hierarchy */
SWIGRUNTIME(swig_type_info *) 
SWIG_TypeDynamicCast(swig_type_info *ty, void **ptr) 
{
  swig_type_info *lastty = ty;
  if (!ty || !ty->dcast) return ty;
  while (ty && (ty->dcast)) {
     ty = (*ty->dcast)(ptr);
     if (ty) lastty = ty;
  }
  return lastty;
}

/* Return the name associated with this type */
SWIGRUNTIME(const char *)
SWIG_TypeName(const swig_type_info *ty) {
  return ty->name;
}

/* Search for a swig_type_info structure */
SWIGRUNTIME(swig_type_info *)
SWIG_TypeQuery(const char *name) {
  swig_type_info *ty = swig_type_list;
  while (ty) {
    if (ty->str && (strcmp(name,ty->str) == 0)) return ty;
    if (ty->name && (strcmp(name,ty->name) == 0)) return ty;
    ty = ty->prev;
  }
  return 0;
}

/* Set the clientdata field for a type */
SWIGRUNTIME(void)
SWIG_TypeClientData(swig_type_info *ti, void *clientdata) {
  swig_type_info *tc, *equiv;
  if (ti->clientdata == clientdata) return;
  ti->clientdata = clientdata;
  equiv = ti->next;
  while (equiv) {
    if (!equiv->converter) {
      tc = swig_type_list;
      while (tc) {
	if ((strcmp(tc->name, equiv->name) == 0))
	  SWIG_TypeClientData(tc,clientdata);
	tc = tc->prev;
      }
    }
    equiv = equiv->next;
  }
}
#endif

#ifdef __cplusplus
}

#endif

/* -----------------------------------------------------------------------------
 * perl5.swg
 *
 * Perl5 runtime library
 * $Header: /cvsroot/devicetool/Solaris-DeviceTree/Libdevinfo-Impl/Impl_wrap.c,v 1.3 2003/09/15 14:41:54 honkbude Exp $
 * ----------------------------------------------------------------------------- */

#define SWIGPERL
#define SWIGPERL5
#ifdef __cplusplus
/* Needed on some windows machines---since MS plays funny
   games with the header files under C++ */
#include <math.h>
#include <stdlib.h>
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

/* Get rid of free and malloc defined by perl */
#undef free
#undef malloc

#ifndef pTHX_
#define pTHX_
#endif

#include <string.h>
#ifdef __cplusplus
}
#endif

/* Macro to call an XS function */

#ifdef PERL_OBJECT 
#define SWIG_CALLXS(_name) _name(cv,pPerl) 
#else 
#ifndef MULTIPLICITY 
#define SWIG_CALLXS(_name) _name(cv) 
#else 
#define SWIG_CALLXS(_name) _name(PERL_GET_THX, cv) 
#endif 
#endif 

/* Macros for low-level exception handling */
#define SWIG_fail      goto fail
#define SWIG_croak(x)  { if ((_swigerr = (const char *) x)) goto fail; }
#define SWIG_MAX_ERRMSG 256

/* Note: SwigMagicFuncHack is a typedef used to get the C++
   compiler to just shut up already */

#ifdef PERL_OBJECT
#define MAGIC_PPERL  CPerlObj *pPerl = (CPerlObj *) this;
typedef int (CPerlObj::*SwigMagicFunc)(SV *, MAGIC *);

#ifdef __cplusplus
extern "C" {
#endif
typedef int (CPerlObj::*SwigMagicFuncHack)(SV *, MAGIC *);
#ifdef __cplusplus
}
#endif

#define SWIG_MAGIC(a,b) (SV *a, MAGIC *b)
#define SWIGCLASS_STATIC
#else
#define MAGIC_PPERL
#define SWIGCLASS_STATIC static
#ifndef MULTIPLICITY
#define SWIG_MAGIC(a,b) (SV *a, MAGIC *b)
typedef int (*SwigMagicFunc)(SV *, MAGIC *);

#ifdef __cplusplus
extern "C" {
#endif
typedef int (*SwigMagicFuncHack)(SV *, MAGIC *);
#ifdef __cplusplus
}
#endif


#else
#define SWIG_MAGIC(a,b) (struct interpreter *interp, SV *a, MAGIC *b)
typedef int (*SwigMagicFunc)(struct interpreter *, SV *, MAGIC *);
#ifdef __cplusplus
extern "C" {
#endif
typedef int (*SwigMagicFuncHack)(struct interpreter *, SV *, MAGIC *);
#ifdef __cplusplus
}
#endif

#endif
#endif

#if defined(WIN32) && defined(PERL_OBJECT) && !defined(PerlIO_exportFILE)
#define PerlIO_exportFILE(fh,fl) (FILE*)(fh)
#endif

/* Modifications for newer Perl 5.005 releases */

#if !defined(PERL_REVISION) || ((PERL_REVISION >= 5) && ((PERL_VERSION < 5) || ((PERL_VERSION == 5) && (PERL_SUBVERSION < 50))))
#ifndef PL_sv_yes
#define PL_sv_yes sv_yes
#endif
#ifndef PL_sv_undef
#define PL_sv_undef sv_undef
#endif
#ifndef PL_na
#define PL_na na
#endif
#endif

#include <stdlib.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifdef SWIG_NOINCLUDE

#ifndef PERL_OBJECT
extern int SWIG_ConvertPtr(SV *, void **, swig_type_info *, int flags);
extern void SWIG_MakePtr(SV *, void *, swig_type_info *, int flags);
#else
extern int _SWIG_ConvertPtr(CPerlObj *, SV *, void **, swig_type_info *,int flags);
extern void _SWIG_MakePtr(CPerlObj *, SV *, void *, swig_type_info *, int flags);
#define SWIG_ConvertPtr(a,b,c,d) _SWIG_ConvertPtr(pPerl,a,b,c,d)
#define SWIG_MakePtr(a,b,c,d) _SWIG_MakePtr(pPerl,a,b,c,d)
#endif

#else

/* Function for getting a pointer value */

#ifndef PERL_OBJECT
SWIGRUNTIME(int) 
SWIG_ConvertPtr(SV *sv, void **ptr, swig_type_info *_t, int flags)
#else
#define SWIG_ConvertPtr(a,b,c,d) _SWIG_ConvertPtr(pPerl,a,b,c,d)
SWIGRUNTIME(int)
_SWIG_ConvertPtr(CPerlObj *pPerl, SV *sv, void **ptr, swig_type_info *_t, int flags)
#endif
{
  char *_c;
  swig_type_info *tc;
  IV   tmp;

  /* If magical, apply more magic */
  if (SvGMAGICAL(sv))
    mg_get(sv);

  /* Check to see if this is an object */
  if (sv_isobject(sv)) {
    SV *tsv = (SV*) SvRV(sv);
    if ((SvTYPE(tsv) == SVt_PVHV)) {
      MAGIC *mg;
      if (SvMAGICAL(tsv)) {
	mg = mg_find(tsv,'P');
	if (mg) {
	  SV *rsv = mg->mg_obj;
	  if (sv_isobject(rsv)) {
	    tmp = SvIV((SV*)SvRV(rsv));
	  }
	}
      } else {
	return -1;
      }
    } else {
      tmp = SvIV((SV*)SvRV(sv));
    }
    if (!_t) {
      *(ptr) = (void *) tmp;
      return 0;
    }
  } else if (! SvOK(sv)) {            /* Check for undef */
    *(ptr) = (void *) 0;
    return 0;
  } else if (SvTYPE(sv) == SVt_RV) {  /* Check for NULL pointer */
    *(ptr) = (void *) 0;
    if (!SvROK(sv)) 
      return 0;
    else
      return -1;
  } else {                            /* Don't know what it is */
      *(ptr) = (void *) 0;
      return -1;
  }
  if (_t) {
    /* Now see if the types match */      
    _c = HvNAME(SvSTASH(SvRV(sv)));
    tc = SWIG_TypeCheck(_c,_t);
    if (!tc) {
      *ptr = (void *) tmp;
      return -1;
    }
    *ptr = SWIG_TypeCast(tc,(void *)tmp);
    return 0;
  }
  *ptr = (void *) tmp;
  return 0;
}
#ifndef PERL_OBJECT
SWIGRUNTIME(void) 
SWIG_MakePtr(SV *sv, void *ptr, swig_type_info *t,int flags)
#else
#define SWIG_MakePtr(a,b,c,d) _SWIG_MakePtr(pPerl,a,b,c,d)
SWIGRUNTIME(void)
_SWIG_MakePtr(CPerlObj *pPerl, SV *sv, void *ptr, swig_type_info *t, int flags)
#endif
{
  sv_setref_pv(sv, (char *) t->name, ptr);
}

#endif

typedef XS(SwigPerlWrapper);
typedef SwigPerlWrapper *SwigPerlWrapperPtr;

/* Structure for command table */
typedef struct {
  const char         *name;
  SwigPerlWrapperPtr  wrapper;
} swig_command_info;

/* Information for constant table */

#define SWIG_INT     1
#define SWIG_FLOAT   2
#define SWIG_STRING  3
#define SWIG_POINTER 4
#define SWIG_BINARY  5

/* Constant information structure */
typedef struct swig_constant_info {
    int              type;
    const char      *name;
    long             lvalue;
    double           dvalue;
    void            *pvalue;
    swig_type_info **ptype;
} swig_constant_info;

#ifdef __cplusplus
}
#endif

/* Structure for variable table */
typedef struct {
  const char   *name;
  SwigMagicFunc   set;
  SwigMagicFunc   get;
  swig_type_info  **type;
} swig_variable_info;

/* Magic variable code */
#ifndef PERL_OBJECT
#define swig_create_magic(s,a,b,c) _swig_create_magic(s,a,b,c)
  #ifndef MULTIPLICITY 
       static void _swig_create_magic(SV *sv, char *name, int (*set)(SV *, MAGIC *), int \
(*get)(SV *,MAGIC *)) { 
  #else 
       static void _swig_create_magic(SV *sv, char *name, int (*set)(struct interpreter*,\
 SV *, MAGIC *), int (*get)(struct interpreter*, SV *,MAGIC *)) { 
  #endif 
#else
#define swig_create_magic(s,a,b,c) _swig_create_magic(pPerl,s,a,b,c)
static void _swig_create_magic(CPerlObj *pPerl, SV *sv, const char *name, int (CPerlObj::*set)(SV *, MAGIC *), int (CPerlObj::*get)(SV *, MAGIC *)) {
#endif
  MAGIC *mg;
  sv_magic(sv,sv,'U',(char *) name,strlen(name));
  mg = mg_find(sv,'U');
  mg->mg_virtual = (MGVTBL *) malloc(sizeof(MGVTBL));
  mg->mg_virtual->svt_get = (SwigMagicFuncHack) get;
  mg->mg_virtual->svt_set = (SwigMagicFuncHack) set;
  mg->mg_virtual->svt_len = 0;
  mg->mg_virtual->svt_clear = 0;
  mg->mg_virtual->svt_free = 0;
}






#ifdef do_open
  #undef do_open
#endif
#ifdef do_close
  #undef do_close
#endif
#ifdef scalar
  #undef scalar
#endif
#ifdef list
  #undef list
#endif
#ifdef apply
  #undef apply
#endif
#ifdef convert
  #undef convert
#endif
#ifdef Error
  #undef Error
#endif
#ifdef form
  #undef form
#endif
#ifdef vform
  #undef vform
#endif
#ifdef LABEL
  #undef LABEL
#endif
#ifdef METHOD
  #undef METHOD
#endif
#ifdef Move
  #undef Move
#endif
#ifdef yylex
  #undef yylex
#endif
#ifdef yyparse
  #undef yyparse
#endif
#ifdef yyerror
  #undef yyerror
#endif
#ifdef invert
  #undef invert
#endif
#ifdef ref
  #undef ref
#endif


/* -------- TYPES TABLE (BEGIN) -------- */

#define  SWIGTYPE_p_dev_t swig_types[0] 
#define  SWIGTYPE_p_di_prom_handle_t swig_types[1] 
#define  SWIGTYPE_p_di_node_t swig_types[2] 
#define  SWIGTYPE_p_ddi_devid_t swig_types[3] 
#define  SWIGTYPE_p_di_prop_t swig_types[4] 
#define  SWIGTYPE_p_p_char swig_types[5] 
#define  SWIGTYPE_p_di_prom_prop_t swig_types[6] 
#define  SWIGTYPE_p_p_uchar_t swig_types[7] 
#define  SWIGTYPE_p_di_minor_t swig_types[8] 
#define  SWIGTYPE_p_void swig_types[9] 
#define  SWIGTYPE_p_p_int swig_types[10] 
#define  SWIGTYPE_p_int swig_types[11] 
static swig_type_info *swig_types[13];

/* -------- TYPES TABLE (END) -------- */

#define SWIG_init    boot_Solaris__DeviceTree__Libdevinfo__Impl

#define SWIG_name   "Solaris::DeviceTree::Libdevinfo::Impl::boot_Solaris__DeviceTree__Libdevinfo__Impl"
#define SWIG_prefix "Solaris::DeviceTree::Libdevinfo::Impl::"

#ifdef __cplusplus
extern "C"
#endif
#ifndef PERL_OBJECT
#ifndef MULTIPLICITY
SWIGEXPORT(void) SWIG_init (CV* cv);
#else
SWIGEXPORT(void) SWIG_init (pTHXo_ CV* cv);
#endif
#else
SWIGEXPORT(void) SWIG_init (CV *cv, CPerlObj *);
#endif


#include <sys/types.h>
#include <sys/mkdev.h>
#include <sys/ddi.h>
#include <libdevinfo.h>

#include <errno.h>

extern di_node_t di_init(char const *,unsigned int);
extern void di_fini(di_node_t);
extern di_node_t di_parent_node(di_node_t);
extern di_node_t di_sibling_node(di_node_t);
extern di_node_t di_child_node(di_node_t);
extern di_node_t di_drv_first_node(char const *,di_node_t);
extern di_node_t di_drv_next_node(di_node_t);
extern di_prop_t di_prop_next(di_node_t,di_prop_t);
extern char *di_prop_name(di_prop_t);
extern int di_prop_type(di_prop_t);
extern dev_t di_prop_devt(di_prop_t);
extern int di_prop_ints(di_prop_t,int **);
extern int di_prop_strings(di_prop_t,char **);
extern int di_prop_bytes(di_prop_t,uchar_t **);
extern di_minor_t di_minor_next(di_node_t,di_minor_t);
extern dev_t di_minor_devt(di_minor_t);
extern char *di_minor_name(di_minor_t);
extern char *di_minor_nodetype(di_minor_t);
extern int di_minor_spectype(di_minor_t);
extern di_prom_handle_t di_prom_init();
extern void di_prom_fini(di_prom_handle_t);
extern di_prom_prop_t di_prom_prop_next(di_prom_handle_t,di_node_t,di_prom_prop_t);
extern char *di_prom_prop_name(di_prom_prop_t);
extern int di_prom_prop_data(di_prom_prop_t,uchar_t **);
extern void sdi_prop_devt(di_prop_t,int *,int *);
extern void devt_majorminor(dev_t,int *,int *);
extern int di_compatible_names(di_node_t,char **);



/* 
char *getErrString() {
  return strerror( errno );
}

int getErrno() {
  return errno;
}
*/


int isDDI_DEV_T_NONE( dev_t devt ) {
  return ( devt == DDI_DEV_T_NONE ? 1 : 0 );
}

char **newStringHandle() {
  return (char **) malloc( sizeof( char * ) );
}

char *getIndexedString( char **stringArray, int index ) {
  char *start = *stringArray;
/*  printf( "Index: %d\n", index ); */

  while( index-- > 0 ) {
    /* start points to the start of the string */
    while( *start != 0 ) {
      start++;
    }
    /* start points to '\0' after the string */
    start++;
    /* start points to first character of new string */
  }

/*  printf( "String: %s\n", start ); */
  return start;
}

void freeStringHandle( char **handle ) {
  free( handle );
}

/* -- int -- */

int **newIntHandle() {
  return (int **) malloc( sizeof( int * ) );
}

int getIndexedInt( int **data, int index ) {
  return (*data)[ index ];
}

void freeIntHandle( int **handle ) {
  free( handle );
}

/* -- uchar_t -- */

uchar_t **newUCharTHandle() {
  return (uchar_t **) malloc( sizeof( uchar_t * ) );
}

int getIndexedByte( uchar_t **data, int index ) {
  int value = (*data)[ index ];
  return value;
}

void freeUCharTHandle( uchar_t **handle ) {
  free( handle );
}

char *UCharTString( uchar_t **data ) {
  return (char *) *data;
}

/* -- -- */

int isDI_NODE_NIL( di_node_t node ) {
  return ( node == DI_NODE_NIL ? 1 : 0 );
}

int isDI_PROP_NIL( di_prop_t prop ) {
  return ( prop == DI_PROP_NIL ? 1 : 0 );
}

void *getProp( di_prop_t prop ) {
}

di_prop_t makeDI_PROP_NIL() {
  return (di_prop_t) NULL;
}

/* -- Minor Node functions -- */

di_minor_t makeDI_MINOR_NIL() {
  return (di_minor_t) DI_MINOR_NIL;
}

int isDI_MINOR_NIL( di_minor_t minor ) {
  return ( minor == DI_MINOR_NIL ? 1 : 0 );
}

/* -- PROM Properties -- */

di_prom_prop_t makeDI_PROM_PROP_NIL() {
  return (di_prom_prop_t) DI_PROM_PROP_NIL;
}

int isDI_PROM_PROP_NIL( di_prom_prop_t minor ) {
  return ( minor == DI_PROM_PROP_NIL ? 1 : 0 );
}

int isDI_PROM_HANDLE_NIL( di_prom_handle_t prom_handle ) {
  return (prom_handle == DI_PROM_HANDLE_NIL ? 1 : 0);
}

int isDDI_DEVID_NIL( ddi_devid_t ddi_devid ) {
  return (ddi_devid == 0 ? 1 : 0);
}


#ifdef PERL_OBJECT
#define MAGIC_CLASS _wrap_Solaris::DeviceTree::Libdevinfo::Impl_var::
class _wrap_Solaris::DeviceTree::Libdevinfo::Impl_var : public CPerlObj {
public:
#else
#define MAGIC_CLASS
#endif
SWIGCLASS_STATIC int swig_magic_readonly(pTHX_ SV *sv, MAGIC *mg) {
    MAGIC_PPERL
    sv = sv; mg = mg;
    croak("Value is read-only.");
    return 0;
}


#ifdef PERL_OBJECT
};
#endif

#ifdef __cplusplus
extern "C" {
#endif
XS(_wrap_di_init) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        char *arg1 ;
        unsigned int arg2 ;
        di_node_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_init(phys_path,flag);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        arg2 = (unsigned int) SvUV(ST(1));
        result = di_init((char const *)arg1,arg2);
        
        {
            di_node_t * resultobj = (di_node_t *) malloc(sizeof(di_node_t));
            memmove(resultobj, &result, sizeof(di_node_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_node_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_fini) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_fini(root);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_fini. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        di_fini(arg1);
        
        
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_parent_node) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        di_node_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_parent_node(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_parent_node. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = di_parent_node(arg1);
        
        {
            di_node_t * resultobj = (di_node_t *) malloc(sizeof(di_node_t));
            memmove(resultobj, &result, sizeof(di_node_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_node_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_sibling_node) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        di_node_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_sibling_node(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_sibling_node. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = di_sibling_node(arg1);
        
        {
            di_node_t * resultobj = (di_node_t *) malloc(sizeof(di_node_t));
            memmove(resultobj, &result, sizeof(di_node_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_node_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_child_node) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        di_node_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_child_node(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_child_node. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = di_child_node(arg1);
        
        {
            di_node_t * resultobj = (di_node_t *) malloc(sizeof(di_node_t));
            memmove(resultobj, &result, sizeof(di_node_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_node_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_drv_first_node) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        char *arg1 ;
        di_node_t arg2 ;
        di_node_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_drv_first_node(drv_name,root);");
        }
        if (!SvOK((SV*) ST(0))) arg1 = 0;
        else arg1 = (char *) SvPV(ST(0), PL_na);
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(1),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_drv_first_node. Expected _p_di_node_t");
            }
            arg2 = *argp;
        }
        result = di_drv_first_node((char const *)arg1,arg2);
        
        {
            di_node_t * resultobj = (di_node_t *) malloc(sizeof(di_node_t));
            memmove(resultobj, &result, sizeof(di_node_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_node_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_drv_next_node) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        di_node_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_drv_next_node(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_drv_next_node. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = di_drv_next_node(arg1);
        
        {
            di_node_t * resultobj = (di_node_t *) malloc(sizeof(di_node_t));
            memmove(resultobj, &result, sizeof(di_node_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_node_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_binding_name) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_binding_name(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_binding_name. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_binding_name(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_bus_addr) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_bus_addr(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_bus_addr. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_bus_addr(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_devid) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        ddi_devid_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_devid(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_devid. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = di_devid(arg1);
        
        {
            ddi_devid_t * resultobj = (ddi_devid_t *) malloc(sizeof(ddi_devid_t));
            memmove(resultobj, &result, sizeof(ddi_devid_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_ddi_devid_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_driver_name) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_driver_name(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_driver_name. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_driver_name(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_state) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        unsigned int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_state(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_state. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (unsigned int)di_state(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDevidNull) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        ddi_devid_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDevidNull(devid);");
        }
        {
            ddi_devid_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_ddi_devid_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDevidNull. Expected _p_ddi_devid_t");
            }
            arg1 = *argp;
        }
        result = (int)isDevidNull(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_driver_ops) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        unsigned int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_driver_ops(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_driver_ops. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (unsigned int)di_driver_ops(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setuv(ST(argvi++), (UV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_instance) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_instance(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_instance. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (int)di_instance(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_nodeid) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_nodeid(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_nodeid. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (int)di_nodeid(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_node_name) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_node_name(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_node_name. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_node_name(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_next) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        di_prop_t arg2 ;
        di_prop_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_prop_next(node,prop);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_next. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(1),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_prop_next. Expected _p_di_prop_t");
            }
            arg2 = *argp;
        }
        result = di_prop_next(arg1,arg2);
        
        {
            di_prop_t * resultobj = (di_prop_t *) malloc(sizeof(di_prop_t));
            memmove(resultobj, &result, sizeof(di_prop_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_prop_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_name) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_prop_name(prop);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_name. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_prop_name(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_type) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_prop_type(prop);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_type. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        result = (int)di_prop_type(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_devt) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        dev_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_prop_devt(prop);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_devt. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        result = di_prop_devt(arg1);
        
        {
            dev_t * resultobj = (dev_t *) malloc(sizeof(dev_t));
            memmove(resultobj, &result, sizeof(dev_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_dev_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_ints) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        int **arg2 = (int **) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_prop_ints(prop,prop_data);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_ints. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        {
            if (SWIG_ConvertPtr(ST(1), (void **) &arg2, SWIGTYPE_p_p_int,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_prop_ints. Expected _p_p_int");
            }
        }
        result = (int)di_prop_ints(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_strings) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        char **arg2 = (char **) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_prop_strings(prop,prop_data);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_strings. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        {
            if (SWIG_ConvertPtr(ST(1), (void **) &arg2, SWIGTYPE_p_p_char,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_prop_strings. Expected _p_p_char");
            }
        }
        result = (int)di_prop_strings(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prop_bytes) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        uchar_t **arg2 = (uchar_t **) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_prop_bytes(prop,prop_data);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prop_bytes. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        {
            if (SWIG_ConvertPtr(ST(1), (void **) &arg2, SWIGTYPE_p_p_uchar_t,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_prop_bytes. Expected _p_p_uchar_t");
            }
        }
        result = (int)di_prop_bytes(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_minor_next) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        di_minor_t arg2 ;
        di_minor_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_minor_next(node,minor);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_minor_next. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        {
            di_minor_t * argp;
            if (SWIG_ConvertPtr(ST(1),(void **) &argp, SWIGTYPE_p_di_minor_t,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_minor_next. Expected _p_di_minor_t");
            }
            arg2 = *argp;
        }
        result = di_minor_next(arg1,arg2);
        
        {
            di_minor_t * resultobj = (di_minor_t *) malloc(sizeof(di_minor_t));
            memmove(resultobj, &result, sizeof(di_minor_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_minor_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_minor_devt) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_minor_t arg1 ;
        dev_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_minor_devt(minor);");
        }
        {
            di_minor_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_minor_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_minor_devt. Expected _p_di_minor_t");
            }
            arg1 = *argp;
        }
        result = di_minor_devt(arg1);
        
        {
            dev_t * resultobj = (dev_t *) malloc(sizeof(dev_t));
            memmove(resultobj, &result, sizeof(dev_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_dev_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_minor_name) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_minor_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_minor_name(minor);");
        }
        {
            di_minor_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_minor_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_minor_name. Expected _p_di_minor_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_minor_name(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_minor_nodetype) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_minor_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_minor_nodetype(minor);");
        }
        {
            di_minor_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_minor_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_minor_nodetype. Expected _p_di_minor_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_minor_nodetype(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_minor_spectype) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_minor_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_minor_spectype(minor);");
        }
        {
            di_minor_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_minor_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_minor_spectype. Expected _p_di_minor_t");
            }
            arg1 = *argp;
        }
        result = (int)di_minor_spectype(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prom_init) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_handle_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: di_prom_init();");
        }
        result = di_prom_init();
        
        {
            di_prom_handle_t * resultobj = (di_prom_handle_t *) malloc(sizeof(di_prom_handle_t));
            memmove(resultobj, &result, sizeof(di_prom_handle_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_prom_handle_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prom_fini) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_handle_t arg1 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_prom_fini(ph);");
        }
        {
            di_prom_handle_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prom_handle_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prom_fini. Expected _p_di_prom_handle_t");
            }
            arg1 = *argp;
        }
        di_prom_fini(arg1);
        
        
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prom_prop_next) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_handle_t arg1 ;
        di_node_t arg2 ;
        di_prom_prop_t arg3 ;
        di_prom_prop_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 3) || (items > 3)) {
            SWIG_croak("Usage: di_prom_prop_next(ph,node,prom_prop);");
        }
        {
            di_prom_handle_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prom_handle_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prom_prop_next. Expected _p_di_prom_handle_t");
            }
            arg1 = *argp;
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(1),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_prom_prop_next. Expected _p_di_node_t");
            }
            arg2 = *argp;
        }
        {
            di_prom_prop_t * argp;
            if (SWIG_ConvertPtr(ST(2),(void **) &argp, SWIGTYPE_p_di_prom_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 3 of di_prom_prop_next. Expected _p_di_prom_prop_t");
            }
            arg3 = *argp;
        }
        result = di_prom_prop_next(arg1,arg2,arg3);
        
        {
            di_prom_prop_t * resultobj = (di_prom_prop_t *) malloc(sizeof(di_prom_prop_t));
            memmove(resultobj, &result, sizeof(di_prom_prop_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_prom_prop_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prom_prop_name) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_prop_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_prom_prop_name(prom_prop);");
        }
        {
            di_prom_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prom_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prom_prop_name. Expected _p_di_prom_prop_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_prom_prop_name(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_prom_prop_data) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_prop_t arg1 ;
        uchar_t **arg2 = (uchar_t **) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_prom_prop_data(prop,prom_prop_data);");
        }
        {
            di_prom_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prom_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_prom_prop_data. Expected _p_di_prom_prop_t");
            }
            arg1 = *argp;
        }
        {
            if (SWIG_ConvertPtr(ST(1), (void **) &arg2, SWIGTYPE_p_p_uchar_t,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_prom_prop_data. Expected _p_p_uchar_t");
            }
        }
        result = (int)di_prom_prop_data(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_devfs_path) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: di_devfs_path(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_devfs_path. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (char *)di_devfs_path(arg1);
        
        {
            ST(argvi) = sv_2mortal(newSVpv( result, 0));
            di_devfs_path_free( result );
            argvi++;
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_sdi_prop_devt) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        int *arg2 = (int *) 0 ;
        int *arg3 = (int *) 0 ;
        int temp2 ;
        int temp3 ;
        int argvi = 0;
        dXSARGS;
        
        arg2 = &temp2;
        arg3 = &temp3;
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: sdi_prop_devt(prop);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of sdi_prop_devt. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        sdi_prop_devt(arg1,arg2,arg3);
        
        
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg2));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg3));
            argvi++;
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_devt_majorminor) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        dev_t arg1 ;
        int *arg2 = (int *) 0 ;
        int *arg3 = (int *) 0 ;
        int temp2 ;
        int temp3 ;
        int argvi = 0;
        dXSARGS;
        
        arg2 = &temp2;
        arg3 = &temp3;
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: devt_majorminor(devt);");
        }
        {
            dev_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_dev_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of devt_majorminor. Expected _p_dev_t");
            }
            arg1 = *argp;
        }
        devt_majorminor(arg1,arg2,arg3);
        
        
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg2));
            argvi++;
        }
        {
            if (argvi >= items) {
                EXTEND(sp,1);
            }
            ST(argvi) = sv_newmortal();
            sv_setiv(ST(argvi),(IV) *(arg3));
            argvi++;
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_di_compatible_names) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        char **arg2 = (char **) 0 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: di_compatible_names(node,OUTPUT);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of di_compatible_names. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        {
            if (SWIG_ConvertPtr(ST(1), (void **) &arg2, SWIGTYPE_p_p_char,0) < 0) {
                SWIG_croak("Type error in argument 2 of di_compatible_names. Expected _p_p_char");
            }
        }
        result = (int)di_compatible_names(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDDI_DEV_T_NONE) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        dev_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDDI_DEV_T_NONE(devt);");
        }
        {
            dev_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_dev_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDDI_DEV_T_NONE. Expected _p_dev_t");
            }
            arg1 = *argp;
        }
        result = (int)isDDI_DEV_T_NONE(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_newStringHandle) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        char **result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: newStringHandle();");
        }
        result = (char **)newStringHandle();
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_p_char,0);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_getIndexedString) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        char **arg1 = (char **) 0 ;
        int arg2 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: getIndexedString(stringArray,index);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_char,0) < 0) {
                SWIG_croak("Type error in argument 1 of getIndexedString. Expected _p_p_char");
            }
        }
        arg2 = (int) SvIV(ST(1));
        result = (char *)getIndexedString(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_freeStringHandle) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        char **arg1 = (char **) 0 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: freeStringHandle(handle);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_char,0) < 0) {
                SWIG_croak("Type error in argument 1 of freeStringHandle. Expected _p_p_char");
            }
        }
        freeStringHandle(arg1);
        
        
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_newIntHandle) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        int **result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: newIntHandle();");
        }
        result = (int **)newIntHandle();
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_p_int,0);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_getIndexedInt) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        int **arg1 = (int **) 0 ;
        int arg2 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: getIndexedInt(data,index);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_int,0) < 0) {
                SWIG_croak("Type error in argument 1 of getIndexedInt. Expected _p_p_int");
            }
        }
        arg2 = (int) SvIV(ST(1));
        result = (int)getIndexedInt(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_freeIntHandle) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        int **arg1 = (int **) 0 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: freeIntHandle(handle);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_int,0) < 0) {
                SWIG_croak("Type error in argument 1 of freeIntHandle. Expected _p_p_int");
            }
        }
        freeIntHandle(arg1);
        
        
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_newUCharTHandle) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        uchar_t **result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: newUCharTHandle();");
        }
        result = (uchar_t **)newUCharTHandle();
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_p_uchar_t,0);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_getIndexedByte) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        uchar_t **arg1 = (uchar_t **) 0 ;
        int arg2 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 2) || (items > 2)) {
            SWIG_croak("Usage: getIndexedByte(data,index);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_uchar_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of getIndexedByte. Expected _p_p_uchar_t");
            }
        }
        arg2 = (int) SvIV(ST(1));
        result = (int)getIndexedByte(arg1,arg2);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_freeUCharTHandle) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        uchar_t **arg1 = (uchar_t **) 0 ;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: freeUCharTHandle(handle);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_uchar_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of freeUCharTHandle. Expected _p_p_uchar_t");
            }
        }
        freeUCharTHandle(arg1);
        
        
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_UCharTString) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        uchar_t **arg1 = (uchar_t **) 0 ;
        char *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: UCharTString(data);");
        }
        {
            if (SWIG_ConvertPtr(ST(0), (void **) &arg1, SWIGTYPE_p_p_uchar_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of UCharTString. Expected _p_p_uchar_t");
            }
        }
        result = (char *)UCharTString(arg1);
        
        ST(argvi) = sv_newmortal();
        if (result) {
            sv_setpv((SV*)ST(argvi++), (char *) result);
        }else {
            sv_setsv((SV*)ST(argvi++), &PL_sv_undef);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDI_NODE_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_node_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDI_NODE_NIL(node);");
        }
        {
            di_node_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_node_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDI_NODE_NIL. Expected _p_di_node_t");
            }
            arg1 = *argp;
        }
        result = (int)isDI_NODE_NIL(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDI_PROP_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDI_PROP_NIL(prop);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDI_PROP_NIL. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        result = (int)isDI_PROP_NIL(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_getProp) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t arg1 ;
        void *result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: getProp(prop);");
        }
        {
            di_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of getProp. Expected _p_di_prop_t");
            }
            arg1 = *argp;
        }
        result = (void *)getProp(arg1);
        
        ST(argvi) = sv_newmortal();
        SWIG_MakePtr(ST(argvi++), (void *) result, SWIGTYPE_p_void,0);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_makeDI_PROP_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prop_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: makeDI_PROP_NIL();");
        }
        result = makeDI_PROP_NIL();
        
        {
            di_prop_t * resultobj = (di_prop_t *) malloc(sizeof(di_prop_t));
            memmove(resultobj, &result, sizeof(di_prop_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_prop_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_makeDI_MINOR_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_minor_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: makeDI_MINOR_NIL();");
        }
        result = makeDI_MINOR_NIL();
        
        {
            di_minor_t * resultobj = (di_minor_t *) malloc(sizeof(di_minor_t));
            memmove(resultobj, &result, sizeof(di_minor_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_minor_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDI_MINOR_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_minor_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDI_MINOR_NIL(minor);");
        }
        {
            di_minor_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_minor_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDI_MINOR_NIL. Expected _p_di_minor_t");
            }
            arg1 = *argp;
        }
        result = (int)isDI_MINOR_NIL(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_makeDI_PROM_PROP_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_prop_t result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 0) || (items > 0)) {
            SWIG_croak("Usage: makeDI_PROM_PROP_NIL();");
        }
        result = makeDI_PROM_PROP_NIL();
        
        {
            di_prom_prop_t * resultobj = (di_prom_prop_t *) malloc(sizeof(di_prom_prop_t));
            memmove(resultobj, &result, sizeof(di_prom_prop_t));
            ST(argvi) = sv_newmortal();
            SWIG_MakePtr(ST(argvi++), (void *) resultobj, SWIGTYPE_p_di_prom_prop_t,0);
        }
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDI_PROM_PROP_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_prop_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDI_PROM_PROP_NIL(minor);");
        }
        {
            di_prom_prop_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prom_prop_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDI_PROM_PROP_NIL. Expected _p_di_prom_prop_t");
            }
            arg1 = *argp;
        }
        result = (int)isDI_PROM_PROP_NIL(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDI_PROM_HANDLE_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        di_prom_handle_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDI_PROM_HANDLE_NIL(prom_handle);");
        }
        {
            di_prom_handle_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_di_prom_handle_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDI_PROM_HANDLE_NIL. Expected _p_di_prom_handle_t");
            }
            arg1 = *argp;
        }
        result = (int)isDI_PROM_HANDLE_NIL(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}


XS(_wrap_isDDI_DEVID_NIL) {
    char _swigmsg[SWIG_MAX_ERRMSG] = "";
    const char *_swigerr = _swigmsg;
    {
        ddi_devid_t arg1 ;
        int result;
        int argvi = 0;
        dXSARGS;
        
        if ((items < 1) || (items > 1)) {
            SWIG_croak("Usage: isDDI_DEVID_NIL(ddi_devid);");
        }
        {
            ddi_devid_t * argp;
            if (SWIG_ConvertPtr(ST(0),(void **) &argp, SWIGTYPE_p_ddi_devid_t,0) < 0) {
                SWIG_croak("Type error in argument 1 of isDDI_DEVID_NIL. Expected _p_ddi_devid_t");
            }
            arg1 = *argp;
        }
        result = (int)isDDI_DEVID_NIL(arg1);
        
        ST(argvi) = sv_newmortal();
        sv_setiv(ST(argvi++), (IV) result);
        XSRETURN(argvi);
        fail:
        (void) _swigerr;
    }
    croak(_swigerr);
}



/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (BEGIN) -------- */

static swig_type_info _swigt__p_dev_t[] = {{"_p_dev_t", 0, "dev_t *", 0},{"_p_dev_t"},{0}};
static swig_type_info _swigt__p_di_prom_handle_t[] = {{"_p_di_prom_handle_t", 0, "di_prom_handle_t *", 0},{"_p_di_prom_handle_t"},{0}};
static swig_type_info _swigt__p_di_node_t[] = {{"_p_di_node_t", 0, "di_node_t *", 0},{"_p_di_node_t"},{0}};
static swig_type_info _swigt__p_ddi_devid_t[] = {{"_p_ddi_devid_t", 0, "ddi_devid_t *", 0},{"_p_ddi_devid_t"},{0}};
static swig_type_info _swigt__p_di_prop_t[] = {{"_p_di_prop_t", 0, "di_prop_t *", 0},{"_p_di_prop_t"},{0}};
static swig_type_info _swigt__p_p_char[] = {{"_p_p_char", 0, "char **", 0},{"_p_p_char"},{0}};
static swig_type_info _swigt__p_di_prom_prop_t[] = {{"_p_di_prom_prop_t", 0, "di_prom_prop_t *", 0},{"_p_di_prom_prop_t"},{0}};
static swig_type_info _swigt__p_p_uchar_t[] = {{"_p_p_uchar_t", 0, "uchar_t **", 0},{"_p_p_uchar_t"},{0}};
static swig_type_info _swigt__p_di_minor_t[] = {{"_p_di_minor_t", 0, "di_minor_t *", 0},{"_p_di_minor_t"},{0}};
static swig_type_info _swigt__p_void[] = {{"_p_void", 0, "void *", 0},{"_p_void"},{0}};
static swig_type_info _swigt__p_p_int[] = {{"_p_p_int", 0, "int **", 0},{"_p_p_int"},{0}};
static swig_type_info _swigt__p_int[] = {{"_p_int", 0, "int *", 0},{"_p_int"},{0}};

static swig_type_info *swig_types_initial[] = {
_swigt__p_dev_t, 
_swigt__p_di_prom_handle_t, 
_swigt__p_di_node_t, 
_swigt__p_ddi_devid_t, 
_swigt__p_di_prop_t, 
_swigt__p_p_char, 
_swigt__p_di_prom_prop_t, 
_swigt__p_p_uchar_t, 
_swigt__p_di_minor_t, 
_swigt__p_void, 
_swigt__p_p_int, 
_swigt__p_int, 
0
};


/* -------- TYPE CONVERSION AND EQUIVALENCE RULES (END) -------- */

static swig_constant_info swig_constants[] = {
{ SWIG_INT,     (char *) SWIG_prefix "NULL", (long) 0, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_NODE_NIL", (long) 0, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_MINOR_NIL", (long) 0, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROM_PROP_NIL", (long) 0, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROM_HANDLE_NIL", (long) 0, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DIIOC", (long) (0xdf<<8), 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DINFOSUBTREE", (long) (0xdf<<8)|0x01, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DINFOMINOR", (long) (0xdf<<8)|0x02, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DINFOPROP", (long) (0xdf<<8)|0x04, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DINFOCPYALL", (long) ((0xdf<<8)|0x01|(0xdf<<8)|0x04|(0xdf<<8)|0x02), 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_DRIVER_DETACHED", (long) 0x8000, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_DEVICE_OFFLINE", (long) 0x1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_DEVICE_DOWN", (long) 0x2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_BUS_QUIESCED", (long) 0x100, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_BUS_DOWN", (long) 0x200, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_BUS_OPS", (long) 0x1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_CB_OPS", (long) 0x2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_STREAM_OPS", (long) 0x4, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PSEUDO_NODEID", (long) -1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_SID_NODEID", (long) -2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROM_NODEID", (long) -3, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROP_TYPE_BOOLEAN", (long) 0, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROP_TYPE_INT", (long) 1, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROP_TYPE_STRING", (long) 2, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROP_TYPE_BYTE", (long) 3, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROP_TYPE_UNKNOWN", (long) 4, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "DI_PROP_TYPE_UNDEF_IT", (long) 5, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "S_IFCHR", (long) 0x2000, 0, 0, 0},
{ SWIG_INT,     (char *) SWIG_prefix "S_IFBLK", (long) 0x6000, 0, 0, 0},
{0}
};
#ifdef __cplusplus
}
#endif
static swig_variable_info swig_variables[] = {
{0}
};
static swig_command_info swig_commands[] = {
{"Solaris::DeviceTree::Libdevinfo::Impl::di_init", _wrap_di_init},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_fini", _wrap_di_fini},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_parent_node", _wrap_di_parent_node},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_sibling_node", _wrap_di_sibling_node},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_child_node", _wrap_di_child_node},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_drv_first_node", _wrap_di_drv_first_node},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_drv_next_node", _wrap_di_drv_next_node},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_binding_name", _wrap_di_binding_name},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_bus_addr", _wrap_di_bus_addr},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_devid", _wrap_di_devid},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_driver_name", _wrap_di_driver_name},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_state", _wrap_di_state},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDevidNull", _wrap_isDevidNull},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_driver_ops", _wrap_di_driver_ops},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_instance", _wrap_di_instance},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_nodeid", _wrap_di_nodeid},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_node_name", _wrap_di_node_name},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_next", _wrap_di_prop_next},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_name", _wrap_di_prop_name},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_type", _wrap_di_prop_type},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_devt", _wrap_di_prop_devt},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_ints", _wrap_di_prop_ints},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_strings", _wrap_di_prop_strings},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prop_bytes", _wrap_di_prop_bytes},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_minor_next", _wrap_di_minor_next},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_minor_devt", _wrap_di_minor_devt},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_minor_name", _wrap_di_minor_name},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_minor_nodetype", _wrap_di_minor_nodetype},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_minor_spectype", _wrap_di_minor_spectype},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prom_init", _wrap_di_prom_init},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prom_fini", _wrap_di_prom_fini},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prom_prop_next", _wrap_di_prom_prop_next},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prom_prop_name", _wrap_di_prom_prop_name},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_prom_prop_data", _wrap_di_prom_prop_data},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_devfs_path", _wrap_di_devfs_path},
{"Solaris::DeviceTree::Libdevinfo::Impl::sdi_prop_devt", _wrap_sdi_prop_devt},
{"Solaris::DeviceTree::Libdevinfo::Impl::devt_majorminor", _wrap_devt_majorminor},
{"Solaris::DeviceTree::Libdevinfo::Impl::di_compatible_names", _wrap_di_compatible_names},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDDI_DEV_T_NONE", _wrap_isDDI_DEV_T_NONE},
{"Solaris::DeviceTree::Libdevinfo::Impl::newStringHandle", _wrap_newStringHandle},
{"Solaris::DeviceTree::Libdevinfo::Impl::getIndexedString", _wrap_getIndexedString},
{"Solaris::DeviceTree::Libdevinfo::Impl::freeStringHandle", _wrap_freeStringHandle},
{"Solaris::DeviceTree::Libdevinfo::Impl::newIntHandle", _wrap_newIntHandle},
{"Solaris::DeviceTree::Libdevinfo::Impl::getIndexedInt", _wrap_getIndexedInt},
{"Solaris::DeviceTree::Libdevinfo::Impl::freeIntHandle", _wrap_freeIntHandle},
{"Solaris::DeviceTree::Libdevinfo::Impl::newUCharTHandle", _wrap_newUCharTHandle},
{"Solaris::DeviceTree::Libdevinfo::Impl::getIndexedByte", _wrap_getIndexedByte},
{"Solaris::DeviceTree::Libdevinfo::Impl::freeUCharTHandle", _wrap_freeUCharTHandle},
{"Solaris::DeviceTree::Libdevinfo::Impl::UCharTString", _wrap_UCharTString},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDI_NODE_NIL", _wrap_isDI_NODE_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDI_PROP_NIL", _wrap_isDI_PROP_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::getProp", _wrap_getProp},
{"Solaris::DeviceTree::Libdevinfo::Impl::makeDI_PROP_NIL", _wrap_makeDI_PROP_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::makeDI_MINOR_NIL", _wrap_makeDI_MINOR_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDI_MINOR_NIL", _wrap_isDI_MINOR_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::makeDI_PROM_PROP_NIL", _wrap_makeDI_PROM_PROP_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDI_PROM_PROP_NIL", _wrap_isDI_PROM_PROP_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDI_PROM_HANDLE_NIL", _wrap_isDI_PROM_HANDLE_NIL},
{"Solaris::DeviceTree::Libdevinfo::Impl::isDDI_DEVID_NIL", _wrap_isDDI_DEVID_NIL},
{0,0}
};

#ifdef __cplusplus
extern "C"
#endif

XS(SWIG_init) {
    dXSARGS;
    int i;
    static int _init = 0;
    if (!_init) {
        for (i = 0; swig_types_initial[i]; i++) {
            swig_types[i] = SWIG_TypeRegister(swig_types_initial[i]);
        }
        _init = 1;
    }
    
    /* Install commands */
    for (i = 0; swig_commands[i].name; i++) {
        newXS((char*) swig_commands[i].name,swig_commands[i].wrapper, (char*)__FILE__);
    }
    
    /* Install variables */
    for (i = 0; swig_variables[i].name; i++) {
        SV *sv;
        sv = perl_get_sv((char*) swig_variables[i].name, TRUE | 0x2);
        if (swig_variables[i].type) {
            SWIG_MakePtr(sv,(void *)1, *swig_variables[i].type,0);
        }else {
            sv_setiv(sv,(IV) 0);
        }
        swig_create_magic(sv, (char *) swig_variables[i].name, swig_variables[i].set, swig_variables[i].get); 
    }
    
    /* Install constant */
    for (i = 0; swig_constants[i].type; i++) {
        SV *sv;
        sv = perl_get_sv((char*)swig_constants[i].name, TRUE | 0x2);
        switch(swig_constants[i].type) {
            case SWIG_INT:
            sv_setiv(sv, (IV) swig_constants[i].lvalue);
            break;
            case SWIG_FLOAT:
            sv_setnv(sv, (double) swig_constants[i].dvalue);
            break;
            case SWIG_STRING:
            sv_setpv(sv, (char *) swig_constants[i].pvalue);
            break;
            case SWIG_POINTER:
            SWIG_MakePtr(sv, swig_constants[i].pvalue, *(swig_constants[i].ptype),0);
            break;
            case SWIG_BINARY:
            /*      obj = SWIG_NewPackedObj(swig_constants[i].pvalue, swig_constants[i].lvalue, *(swig_constants[i].ptype)); */
            break;
            default:
            break;
        }
        SvREADONLY_on(sv);
    }
    
    ST(0) = &PL_sv_yes;
    XSRETURN(1);
}

